
#include "TFile.h"
#include "TTree.h"
#include "TH1D.h"
#include "TH2D.h"
#include "TH1I.h"
#include "TF1.h"
#include "TProfile2D.h"
#include "TGraph.h"
#include "TGraph2D.h"
#include "TGraphErrors.h"
#include "TGaxis.h"
#include "TMath.h"
#include "TRandom3.h"
#include "TCanvas.h"
#include "TLegend.h"
#include <iostream>
#include <fstream>
#include <vector>
#include "string.h"
#include <sstream>

using namespace std;

//----------------------------------------------------------------------------------------------

//rescale function
void rescaleXaxis(TH1D* h, Double_t ScaleFactor)
{
	Double_t Xi = 0;
	Char_t binlabel[128]; 
	Int_t bin_index = 0;
	Int_t i = 0;
		
	Double_t step = 1/ScaleFactor;
  	Double_t nbins = h->GetSize()-2;
  	TAxis* xax = h->GetXaxis();	
  	Double_t dX = (xax->GetXmax()-xax->GetXmin())/nbins;
  	
	while (i*step <= xax->GetXmax()-1) {
    	bin_index = xax->FindBin(i*step);
    	Xi = xax->GetXmin()+(bin_index+0.5)*dX;
    	sprintf(binlabel,"%.0f",Xi*ScaleFactor);
    	xax->SetBinLabel(bin_index,binlabel);
    	i+=1;
    }
}

//----------------------------------------------------------------------------------------------

//main
void SDanalysis()
{	
	gROOT->Reset();

	//----------------------------------input--------------------------------
  	//open an input file 
  	TFile* file = TFile::Open("output/output.root");
  	
  	//set Histogram parameters
  	Int_t nbinsE = 100; 		
  	Double_t Emin = 0;              //keV
  	Double_t Emax = 100; 			//keV
  	
  	Double_t binXsize = 3.; 		//mm
  	Double_t binXStart = -150.;		//mm
  	Double_t binXEnd = -binXStart; 
  	Int_t nbinsX = binXEnd*2./binXsize; 
  	
  	Double_t binYsize = 3.; 		//mm   
  	Double_t binYStart = -150.;		//mm
  	Double_t binYEnd = binXEnd; 
  	Int_t nbinsY = binYEnd*2./binYsize;	
  	
  	//options
  	Bool_t IWantSpectrum = true;
  	Bool_t IWantPlot2D = true;
  	 	 	  	
  	//disable statistics
  	gStyle->SetOptStat(0);
  	
  	//export root file to a text file	
  	Bool_t IwantExport = false;
  	Char_t ExportFileName[128];
  	sprintf(ExportFileName,"output/output.dat");	
  	//----------------------------------------------------------------------- 	

  	
  	//original tree extraction
  	Double_t x,y,E,W;
  	Int_t id,nri,nci;  	
  	TTree* t1 = (TTree*)file->Get("ntuple_virtual_screen");
  	t1->SetBranchAddress("x",&x);
   	t1->SetBranchAddress("y",&y);
   	t1->SetBranchAddress("E",&E);
    Int_t N = t1->GetEntries();
    cout << endl << "N: " << N << endl << endl;
    const Int_t Nval = N;
    
  	
       	 	
   	//spectrum
   	if (IWantSpectrum) {
	  	TH1D* Spectrum = new TH1D("Spectrum", "", nbinsE, Emin, Emax);
	  	for (Int_t i=0; i<N; i++) {
   			t1->GetEntry(i);
      		Spectrum->Fill(E,1);
   		}  
	  	Spectrum->SetLineColor(kBlack);
	  	Spectrum->SetLineWidth(2); 
	  	Spectrum->SetXTitle("Energy (keV)");
	  	Spectrum->GetXaxis()->CenterTitle();
	  	Spectrum->GetXaxis()->SetTitleOffset(1.2); 
	  	Spectrum->SetYTitle("Counts");
	  	Spectrum->GetYaxis()->CenterTitle();
	  	Spectrum->GetYaxis()->SetTitleOffset(1.3); 	 
	  	TCanvas* c1 = new TCanvas("c3","",1200,800); 	 	
	  	c1->cd(); 
	  	Spectrum->Draw("h"); 	
	  	
	  	Int_t binpeak = Spectrum->GetMaximumBin(); 
	  	Double_t Epeak = Spectrum->GetXaxis()->GetBinCenter(binpeak); 	
	  	cout << "peak energy: " << Epeak << " keV" << endl;
	  	cout << "mean energy: " << Spectrum->GetMean() << " keV" << endl;
	  	cout << "BW of spectrum: " 
		     << Spectrum->GetRMS()*100/Spectrum->GetMean() << "%" 
		     << endl << endl;
    }
  	
  	
  	//spatial distribution
	TH2D* SpatialDistribution = new TH2D("Idistrib", "", nbinsX,
														 binXStart, 
														 binXEnd, 
														 nbinsY, 
													     binYStart, 
														 binYEnd); 
	
	TProfile2D* Eprofile = new TProfile2D("Edistrib", "", nbinsX, 
	                                                      binXStart, 
	                                                      binXEnd, 
	                                                      nbinsY, 
	                                                      binYStart, 
	                                                      binYEnd);	
	
	if (IWantPlot2D) {
		//impinging photon distribution
		for (Int_t i=0; i<N; i++) {
   		   	t1->GetEntry(i);
			SpatialDistribution->Fill(x,y,1);
      	} 
		SpatialDistribution->SetXTitle("x (mm)");
		SpatialDistribution->GetXaxis()->CenterTitle();
		SpatialDistribution->GetXaxis()->SetTitleOffset(1.2); 
		SpatialDistribution->SetYTitle("y (mm)");
		SpatialDistribution->GetYaxis()->CenterTitle();
		SpatialDistribution->GetYaxis()->SetTitleOffset(1.3); 
		TCanvas* c2 = new TCanvas("c4","",800,800); 
		//c2->Divide(2,1); 
		c2->cd(1);
		SpatialDistribution->SetContour(50);
		SpatialDistribution->Draw("colz");
		SpatialDistribution->GetZaxis()->SetTitle("Counts (arb. units)");
		SpatialDistribution->GetZaxis()->SetTitleOffset(-1.3); 
		c2->SetRealAspectRatio(1.);
		//SpatialDistribution->GetXaxis()->SetRangeUser(-100,);
	}
  	
  	
  	//export to a text file
  	if (IwantExport) {
  		//open a text file
  		ofstream f(ExportFileName); 
    	if (!f) {
        	cout << "Error opening the file!";
        	return;
    	}
    	for (Int_t i=0; i<Nval; i++) {
    		t1->GetEntry(i); 
			f << x << " " 
			  << y << " " 
			  << E << endl;	
    	} 
  		//close the text file
    	f.close(); 
   		cout << "Export of the root file to a text file successful!" << endl << endl;
   	}

}

