//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
//
//
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#ifndef DetectorConstruction_h
#define DetectorConstruction_h 1
#endif

#include "G4VUserDetectorConstruction.hh"

#include "G4Box.hh"
#include "G4Tubs.hh"
#include "G4SubtractionSolid.hh"

#include "G4LogicalVolume.hh"
#include "G4VPhysicalVolume.hh"
#include "G4RunManager.hh"

#include "globals.hh"


//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

class DetectorConstruction : public G4VUserDetectorConstruction
{
public:   
    DetectorConstruction();
    ~DetectorConstruction();
    
    G4VPhysicalVolume* Construct();
    void ConstructSDandField();
    
    //custom methods   
    void DefineMaterials();
    G4double GetA() const {return fa;}
    G4double GetB() const {return fb;}
	G4double GetC() const {return fc;}
	
	G4LogicalVolume* GetMyVolume() const {return myLogicalVolume;}
	
	G4int GetNumberOfScoringVolumes() const {return fNumberOfScoringVolumes;}
	

private:
	//definition of Materilas
	G4Material* Vacuum;
	G4Material* Air;
	G4Material* Water;
	G4Material* Lead;
	
	//definiton of geometrical variables
	G4double fa; //width of myVolume
	G4double fb; //height of myVolume
	G4double fc; //thickness of myVolume

	//definition of Volumes
	G4LogicalVolume* myLogicalVolume;	
	//G4LogicalVolume* myPolyhedraLogical;
	//G4LogicalVolume* myPolyhedraLogical2;
	G4int static const fNumberOfScoringVolumes = 2;
	G4LogicalVolume* fScoringVolume[fNumberOfScoringVolumes];
	
	//boolean variable to activate the magnetic field
	G4bool fSetMagneticField = true;
};


