//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
//
//
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#ifndef PhysicsList_h
#define PhysicsList_h 1

#include "globals.hh"
#include "G4VUserPhysicsList.hh"
#include "G4VModularPhysicsList.hh"
#include <vector>

class G4VPhysicsConstructor;
class PhysicsListMessenger;

class G4Cerenkov;
class G4Scintillation;
class G4OpAbsorption;
class G4OpRayleigh;
class G4OpMieHG;
class G4OpBoundaryProcess;

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

class PhysicsList : public G4VUserPhysicsList
{
public:
	PhysicsList();
    virtual ~PhysicsList();

public:
    virtual void ConstructParticle();
    virtual void ConstructProcess();
    virtual void SetCuts();

	//custom methods
    void ConstructDecay();
    void ConstructOp();
    
    //for the Messenger 
    void SetVerbose(G4int);
    void SetNbOfPhotonsCerenkov(G4int);   
    void SetOpPhotonPhysics(G4int);
    void SetDefaultCutsValue(G4double);
    void SelectPhysicsList(const G4String& name);
    void SetCerenkovProcess(G4int);
    void SetHadExtraPhysics(G4bool bVar) {fIWantHadExtraPhysics = bVar;}
 
private:
    PhysicsList & operator = (const PhysicsList &right);
    PhysicsList(const PhysicsList&);
    
    G4VPhysicsConstructor* fParticleList; 
    G4VPhysicsConstructor* fEmPhysicsList;    
    G4VPhysicsConstructor* fEmExtraPhysicsList;
    G4VPhysicsConstructor* fHadPhysicsList;
    G4VPhysicsConstructor* fHadElPhysicsList;
    G4VPhysicsConstructor* fIonPhysicsList;
    G4VPhysicsConstructor* fStoppingPhysicsList; 
    
    G4bool fIWantHadExtraPhysics;
         
    G4bool fIWantOpticalPhotons;
    G4bool fIWantCerernkov;

	G4int fOPVerboseLevel;
	G4int fMaxNumPhotonStep;

    static G4ThreadLocal G4Cerenkov* fCerenkovProcess;
    static G4ThreadLocal G4Scintillation* fScintillationProcess;
    static G4ThreadLocal G4OpAbsorption* fAbsorptionProcess;
    static G4ThreadLocal G4OpRayleigh* fRayleighScatteringProcess;
    static G4ThreadLocal G4OpMieHG* fMieHGScatteringProcess;
    static G4ThreadLocal G4OpBoundaryProcess* fBoundaryProcess;
    
    PhysicsListMessenger* fPMessenger;
};

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#endif 

