
#include "TFile.h"
#include "TTree.h"
#include "TH1D.h"
#include "TH2D.h"
#include "TH1I.h"
#include "TF1.h"
#include "TProfile2D.h"
#include "TGraph.h"
#include "TGraphErrors.h"
#include "TGaxis.h"
#include "TAxis.h"
#include "TMath.h"
#include "TRandom3.h"
#include "TCanvas.h"
#include "TLegend.h"
#include <iostream>
#include <fstream>
#include <vector>
#include "string.h"

using namespace std;

void spectrumAnalysis()
{	
	gROOT->Reset();
  	 	
  	//----------------------------------input--------------------------------  	  	 	 	
  	//open a simulation result file 
    TFile* file = TFile::Open("output/output.root");
    
    //graph parameters
    Double_t NbinE = 100;
    Double_t Emin = 0.; 	
    Double_t Emax = 100.;	//keV
    //-----------------------------------------------------------------------
        
    //Energy Deposition
  	TH1D* EnergyDep = new TH1D("EnergyDep", "", NbinE, Emin, Emax);
  	TTree* myTree = (TTree*)file->Get("ntuple_edep");
	myTree->Project("EnergyDep", "Edep"); 	
  	EnergyDep->SetLineColor(kRed); 
  	EnergyDep->SetLineWidth(2);
  	EnergyDep->SetXTitle("E (keV)");
  	EnergyDep->GetXaxis()->CenterTitle();
  	EnergyDep->GetXaxis()->SetTitleOffset(1.1); 
  	EnergyDep->SetYTitle("Counts");
  	EnergyDep->GetYaxis()->CenterTitle();
  	EnergyDep->GetYaxis()->SetTitleOffset(1.6); 	
  	EnergyDep->SetTitle("Spectrum of energy deposited inside my volume"); 
  	TCanvas* c1 = new TCanvas("c1","",1200,800); 	
  	c1->cd(); 						
  	//gPad->SetLogy();				
  	EnergyDep->Draw();
  		    
    //original tree extraction
  	Double_t edep;
	Int_t N = myTree->GetEntries();
	myTree->SetBranchAddress("Edep",&edep);
    const Int_t Nsample = N;
    Double_t EdepArray[Nsample];
    Double_t totEdep = 0.;
    Double_t totEdepCorr = 0.;
    for (Int_t i=0; i<Nsample; i++) {
    	myTree->GetEntry(i); 
    	EdepArray[i] = edep;
    	totEdep += edep;
    } 
    
    cout << endl;
    cout << "total energy deposited: " << totEdep*0.001 << " MeV" << endl; 
    cout << endl;   	
}

